#ifndef FIREWALL_TYPES_H
#define FIREWALL_TYPES_H

#include "stdint.h"

static const uint8_t IPV4ADDRESS_LENGTH = 16;

typedef enum firewall_target : uint8_t
{
    FW_REJECT = 0,
    FW_DROP = 1,
    FW_ACCEPT = 2,
} firewall_target_t;

typedef enum firewall_protocol : uint8_t
{
    FW_TCP = 0,
    FW_UDP = 1,
    FW_ALL = 255,
} firewall_protocol_t;

typedef struct firewall_rule
{
    uint8_t key;
    char source[IPV4ADDRESS_LENGTH];
    char destination[IPV4ADDRESS_LENGTH];
    firewall_protocol_t protocol;
    firewall_target_t target;
    struct firewall_rule *next;
} firewall_rule_t;

String protocol_to_string(firewall_protocol_t &protocol)
{
    switch (protocol)
    {
    case FW_TCP:
        return "TCP";
    case FW_UDP:
        return "UDP";
    default:
        return "ALL";
    }
}

firewall_protocol_t string_to_protocol(std::string &protocol)
{
    if (protocol.compare("TCP") == 0)
        return FW_TCP;
    else if (protocol.compare("UDP") == 0)
        return FW_UDP;
    else
        return FW_ALL;
}

String target_to_string(firewall_target_t &target)
{
    switch (target)
    {
    case FW_REJECT:
        return "REJECT";
    case FW_DROP:
        return "DROP";
    default:
        return "ACCEPT";
    }
}

firewall_target_t string_to_target(std::string &target)
{
    if (target.compare("REJECT") == 0)
        return FW_REJECT;
    else if (target.compare("DROP") == 0)
        return FW_DROP;
    else
        return FW_ACCEPT;
}

#endif
